#
# ldaprpt.awk
#
# interpret the output of ipreport to pull out the time and operation
#
# usage : ipreport tracefile | awk -f ldaprpt.awk
#
BEGIN {
    operation["60"]="BindRequest";
    operation["61"]="BindResponse";
    operation["42"]="UnbindRequest";
    operation["63"]="SearchRequest";
    operation["64"]="SearchResponse";
    operation["65"]="SearchResult";
    operation["66"]="ModifyRequest";
    operation["67"]="ModifyResponse";
    operation["68"]="AddRequest";
    operation["69"]="AddResponse";
    operation["4a"]="DelRequest";
    operation["6b"]="DelResponse";
    operation["6c"]="ModifyRDNRequest";
    operation["6d"]="ModifyRDNResponse";
    operation["6e"]="CompareRequest";
    operation["6f"]="CompareResponse";
    operation["50"]="AbandonRequest";
    scope[0]="baseObject";
    scope[1]="singleLevel";
    scope[2]="wholeSubtree";
    modop[0]="add";
    modop[1]="delete";
    modop[2]="replace";
    derefAliases[0]="neverDerefAliases";
    deferAliases[1]="derefInSearching";
    derefAliases[2]="derefFindingBaseObj";
    derefAliases[3]="derefAlways";
    resultCode[0]="success"
    resultCode[1]="operationsError"
    resultCode[2]="protocolError"
    resultCode[3]="timeLimitExceeded"
    resultCode[4]="sizeLimitExceeded"
    resultCode[5]="compareFalse"
    resultCode[6]="compareTrue"
    resultCode[7]="authMethodNotSupported"
    resultCode[8]="strongAuthRequired"
    resultCode[10]="referral"
    resultCode[11]="adminLimitExceeded"
    resultCode[12]="unavailableCriticalExtension"
    resultCode[13]="confidentialityRequired"
    resultCode[14]="saslBindInProgress"
    resultCode[16]="noSuchAttribute"
    resultCode[17]="undefinedAttributeType"
    resultCode[18]="inappropriateMatching"
    resultCode[19]="constraintViolation"
    resultCode[20]="attributeOrValueExists"
    resultCode[21]="invalidAttributeSyntax"
    resultCode[32]="noSuchObject"
    resultCode[33]="aliasProblem"
    resultCode[35]="invalidDNSyntax"
    resultCode[36]="aliasDereferencingProblem"
    resultCode[48]="inappropriateAuthentication"
    resultCode[49]="invalidCredentials"
    resultCode[50]="insufficientAccessRights"
    resultCode[51]="busy"
    resultCode[52]="unavailable"
    resultCode[53]="unwillingToPerform"
    resultCode[54]="loopDetect"
    resultCode[64]="namingViolation"
    resultCode[65]="objectClassViolation"
    resultCode[66]="notAllowedOnNonLeaf"
    resultCode[67]="notAllowedOnRDN"
    resultCode[68]="entryAlreadyExists"
    resultCode[69]="objectClassModsProhibited"
    resultCode[71]="affectsMultipleDSAs"
    resultCode[80]="other"
}
/^====/ {
    packetsize=$2;
    timestamp=$9;
    n=split(timestamp,times,":");
    packettime=times[1]*3600 + times[2]*60 + times[3]
    in_ldap_packet = 0;
    PrintOp();
}

function PrintOp(){
    if (string_part != "") {
       hexptr += 2;  # skip opcode
       bodysize = getL();  # size of body
       if (operation[opcode]=="BindRequest") {
           BindRequest();
       } else {
         if (operation[opcode]=="BindResponse") {
             BindResponse();
         } else {
           if (operation[opcode]=="UnbindRequest") {
               UnbindRequest();
           } else {
             if (operation[opcode]=="SearchRequest") {
                 SearchRequest();
             } else {
               if (operation[opcode]=="SearchResponse") {
#                   SearchResponse();
               } else {
                 if (operation[opcode]=="SearchResult") {
                     SearchResult();
                 } else {
                   if (operation[opcode]=="AddRequest") {
                       AddRequest();
                   } else {
                     if (operation[opcode]=="AddResponse") {
                         AddResponse();
                     } else {
                       if (operation[opcode]=="ModifyRequest") {
                           ModifyRequest();
                       } else {
                         if (operation[opcode]=="ModifyResponse") {
                             ModifyResponse();
                         } else {
                           if (operation[opcode]=="DelRequest") {
                               DelRequest();
                           } else {
                             if (operation[opcode]=="DelResponse") {
                                 DelResponse();
                             } else {
                               if (operation[opcode]=="CompareRequest") {
                                   CompareRequest();
                               } else {
                                 if (operation[opcode]=="CompareResponse") {
                                     CompareResponse();
                                 } else {
                                   print bodysize, substr(hex,hexptr);
                                   print "	" substr(string_part,hexptr/2);
                                 }
                               }
                             }
                           }
                         }
                       }
                     }
                   }
                 }
               }
             }
           }
         }
       }
       string_part = "";
    }
}
/source port=389/ {
    direction="response";
    port=substr($4,6);
}
/destination port=389/ {
    direction="request";
    port=substr($2,6);
    port=substr(port,0,length(port)-1);
}
/^00000000/ {
    if (substr($2,1,2)=="30") {
        hex=$2 $3 $4 $5;
        hexptr=3;   # skip tag
        n=getL();   # length of message
        seqid = getInt();
        uniqseqid = port "." seqid;
        opcode = substr(hex,hexptr,2);
	delay = "";
	if (direction == "request") {
	    requesttime[uniqseqid] = packettime;
        } else {
	    responsetime[uniqseqid] = packettime;
            if (requesttime[uniqseqid] > 0) {
	       delay = responsetime[uniqseqid] - requesttime[uniqseqid] " seconds";
            }
	}
        print timestamp, "port: " port, "seq: " seqid, operation[opcode], delay;
        in_ldap_packet = 1;
        string_part = "";
    }
}
/\*\*\*\*\*\*\*\*/ {
    $0="000001a0     00000000 00000000 00000000 00000000     |................|";
}
/^000/ {
    if (in_ldap_packet == 1) {
        string_part = string_part substr($0,55,16);
        if ($1 != "00000000") {
            $0 = substr($0,1,50);
            hex= hex $2 $3 $4 $5;
        }
    }
}
END {
    if (string_part != "") {
       PrintOp();
    }
}

function BindRequest(version,name) {
   #
   # print out parameters to bind request
   #
   #    BindRequest ::= [APPLICATION 0] SEQUENCE {
   #            version                 INTEGER (1 .. 127),
   #            name                    LDAPDN,
   #            authentication          AuthenticationChoice }
   #

        version = getInt();
        printf "version: " version;
        name = getString();
        if (name == "") {
           printf " anonymous";
           hexptr += 4;   # skip AuthenticationChoice
        } else {
           printf " name: \"" name "\" password: \"" getString() "\"";
        }
        printf "\n";
}

function BindResponse() {
   #
   # print out parameters to bind response
   #
   #    BindResponse ::= [APPLICATION 1] SEQUENCE {
   #         COMPONENTS OF LDAPResult,
   #         serverSaslCreds    [7] OCTET STRING OPTIONAL }
   #
        LDAPResult();
        printf "\n";
}

function UnbindRequest() {
   #
   # print out parameters to unbind response
   #
   #   UnbindRequest ::= [APPLICATION 2] NULL
   #
}

function LDAPResult(resultcode, matchedDN, errormessage) {
   #    LDAPResult ::= SEQUENCE {
   #            resultCode      ENUMERATED {
   #                         success                      (0),
   #                         operationsError              (1),
   #                         protocolError                (2),
   #                         timeLimitExceeded            (3),
   #                         sizeLimitExceeded            (4),
   #                         compareFalse                 (5),
   #                         compareTrue                  (6),
   #                         authMethodNotSupported       (7),
   #                         strongAuthRequired           (8),
   #                                    -- 9 reserved --
   #                         referral                     (10),  -- new
   #                         adminLimitExceeded           (11),  -- new
   #                         unavailableCriticalExtension (12),  -- new
   #                         confidentialityRequired      (13),  -- new
   #                         saslBindInProgress           (14),  -- new
   #                         noSuchAttribute              (16),
   #                         undefinedAttributeType       (17),
   #                         inappropriateMatching        (18),
   #                         constraintViolation          (19),
   #                         attributeOrValueExists       (20),
   #                         invalidAttributeSyntax       (21),
   #                                    -- 22-31 unused --
   #                         noSuchObject                 (32),
   #                         aliasProblem                 (33),
   #                         invalidDNSyntax              (34),
   #                         -- 35 reserved for undefined isLeaf --
   #                         aliasDereferencingProblem    (36),
   #                                    -- 37-47 unused --
   #                         inappropriateAuthentication  (48),
   #                         invalidCredentials           (49),
   #                         insufficientAccessRights     (50),
   #                         busy                         (51),
   #                         unavailable                  (52),
   #                         unwillingToPerform           (53),
   #                         loopDetect                   (54),
   #                                    -- 55-63 unused --
   #                         namingViolation              (64),
   #                         objectClassViolation         (65),
   #                         notAllowedOnNonLeaf          (66),
   #                         notAllowedOnRDN              (67),
   #                         entryAlreadyExists           (68),
   #                         objectClassModsProhibited    (69),
   #                                    -- 70 reserved for CLDAP --
   #                         affectsMultipleDSAs          (71), -- new
   #                                    -- 72-79 unused --
   #                         other                        (80) },
   #                         -- 81-90 reserved for APIs --
   #            matchedDN       LDAPDN,
   #            errorMessage    LDAPString,
   #            referral        [3] Referral OPTIONAL }
     
        resultcode = getInt();
        printf "resultCode: " resultCode[resultcode] " (" resultcode ")";
        matchedDN = getString();
        if ( matchedDN != "") {  
           printf " matchedDN: " matchedDN;
        }
        errormessage = getString();
        if (errormessage != "") {
           printf " errorMessage: " errormessage;
        }
# handle optional referral 
        if ((hexptr < length(hex))&&(substr(hex,hexptr,2)=="a3")) {
           hexptr += 4;
           referral = getString();
           if (referral != "") {
              printf " referral: " referral;
           }
        }
}

function SearchRequest() {
   #
   # print out parameters to search request
   #
   #    SearchRequest ::= [APPLICATION 3] SEQUENCE {
   #            baseObject      LDAPDN,
   #            scope           ENUMERATED {
   #                    baseObject              (0),
   #                    singleLevel             (1),
   #                    wholeSubtree            (2) },
   #            derefAliases    ENUMERATED {
   #                    neverDerefAliases       (0),
   #                    derefInSearching        (1),
   #                    derefFindingBaseObj     (2),
   #                    derefAlways             (3) },
   #            sizeLimit       INTEGER (0 .. maxInt),
   #            timeLimit       INTEGER (0 .. maxInt),
   #            typesOnly       BOOLEAN,
   #            filter          Filter,
   #            attributes      AttributeDescriptionList }

        printf "base: \"" getString() "\"";
        printf " scope: " scope[getInt()];
        printf " derefAliases: " derefAliases[getInt()];
        printf " sizelimit: " getInt();
        printf " timelimit: " getInt();
        printf " typesOnly: " getBool();
        printf " filter: ";
        printf "\"";
        printFilter();
        printf "\"";
        printAttributes();
        print " "
        hexptr -= 2;
}

function SearchResponse() {
   #
   # print out parameters to search response
   #
   #    SearchResultEntry ::= [APPLICATION 4] SEQUENCE {
   #            objectName      LDAPDN,
   #            attributes      PartialAttributeList }
   #
   #    PartialAttributeList ::= SEQUENCE OF SEQUENCE {
   #            type    AttributeDescription,
   #            vals    SET OF AttributeValue }
   #
   #    AttributeDescription ::= LDAPString
   #
   #    AttributeValue ::= OCTET STRING

        print "objectName: \"" getString() "\"";
        hexptr += 2;     skip tag - sequence
        size = getL();   length of sequence
        if (size != 0) {
            attrlistend = hexptr+size*2;
            if (attrlistend > length(hex)*2) attrlistend = length(hex)*2;
            while(hexptr < attrlistend) {    # iterate through sequence
                hexptr += 2;     skip tag - sequence
                avlsize = getL();   length of sequence
                attrName = getString();
                hexptr += 2;     skip tag - set
                avlsize = getL();   length of set
                avlend = hexptr + avlsize*2;
                if (avlend > length(hex)*2) avlend = length(hex)*2;
                while(hexptr < avlend) {    # iterate through set
                    print attrName "=" getString();
                }
            }
        }   
        print substr(hex,hexptr);
}

function SearchResult() {
   #
   # print out parameters to search result
   #
   #    SearchResultDone ::= [APPLICATION 5] LDAPResult
   #
        LDAPResult();
        printf "\n";
}

function CompareRequest() {
   #
   # print out parameters to search result
   #
   #    CompareRequest ::= [APPLICATION 14] SEQUENCE {
   #          entry             LDAPDN,
   #          AttributeValueAssertion }
   #
        print "objectName: \"" getString() "\"";
        hexptr += 2;     skip tag - sequence
        size = getL();   length of sequence
        if (size != 0) {
          printf "(" getString() "=" getString() ")" ;
        }   
        print substr(hex,hexptr);
}

function CompareResponse() {
   #
   # print out parameters to search result
   #
   #    CompareResponse ::= [APPLICATION 15] LDAPResult
   #
        LDAPResult();
        printf "\n";
}

function AddRequest() {
   #
   # print out parameters to add request
   #
   #    AddRequest ::= [APPLICATION 8] SEQUENCE {
   #            entry           LDAPDN,
   #            attributes      AttributeList }
   #
   #    AttributeList ::= SEQUENCE OF SEQUENCE {
   #            type    AttributeDescription,
   #            vals    SET OF AttributeValue }
   #
        SearchResponse();
}

function AddResponse() {
   #
   # print out parameters to add response
   #
   #   AddResponse ::= [APPLICATION 9] LDAPResult
   #
        LDAPResult();
        printf "\n";
}

function ModifyRequest() {
   #
   # print out parameters to modify request
   #
   #    ModifyRequest ::= [APPLICATION 6] SEQUENCE {
   #            object          LDAPDN,
   #            modification    SEQUENCE OF SEQUENCE {
   #                    operation       ENUMERATED {
   #                                            add     (0),
   #                                            delete  (1),
   #                                            replace (2) },
   #                    modification    AttributeTypeAndValues } }
   #
   #    AttributeTypeAndValues ::= SEQUENCE {
   #            type    AttributeDescription,
   #            vals    SET OF AttributeValue }
   #
        print "objectName: \"" getString() "\"";
        hexptr += 2;     skip tag - sequence
        size = getL();   length of sequence
        if (size != 0) {
            attrlistend = hexptr+size*2;
            while(hexptr < attrlistend) {    # iterate through sequence
                hexptr += 2;     skip tag - sequence
                avlsize = getL();   length of sequence
                printf " operation: " modop[getInt()] " ";
                hexptr += 2;     skip tag - sequence
                avlsize = getL();   length of sequence
                attrName = getString();
                hexptr += 2;     skip tag - set
                avlsize = getL();   length of set
                avlend = hexptr + avlsize*2;
                while(hexptr < avlend) {    # iterate through set
                    print attrName "=" getString();
                }
            }
        }   
        print substr(hex,hexptr);
}

function ModifyResponse() {
   #
   # print out parameters to modify response
   #
   #    ModifyResponse ::= [APPLICATION 7] LDAPResult
   #
        LDAPResult();
        printf "\n";
}

function DelRequest() {
   #
   # print out parameters to add request
   #
   #    DelRequest ::= [APPLICATION 10] LDAPDN
   #
        hexptr -= 4;
        print "objectName: \"" getString() "\"";
}

function DelResponse() {
   #
   # print out parameters to add response
   #
   #   DelResponse ::= [APPLICATION 11] LDAPResult
   #
        LDAPResult();
        printf "\n";
}

function getString(size,stringptr) {
        hexptr += 2;  # skip tag
        size = getL();  # size of string
        stringptr = hexptr/2+1;  # beginning of string
        hexptr += size*2;  # skip string
        return substr(string_part,stringptr,size);
}

function printFilter(filtertype,filterlength,filterend) {
   #
   #    Filter ::= CHOICE {
   #            and             [0] SET OF Filter,
   #            or              [1] SET OF Filter,
   #            not             [2] Filter,
   #            equalityMatch   [3] AttributeValueAssertion,
   #            substrings      [4] SubstringFilter,
   #            greaterOrEqual  [5] AttributeValueAssertion,
   #            lessOrEqual     [6] AttributeValueAssertion,
   #            present         [7] AttributeDescription,
   #            approxMatch     [8] AttributeValueAssertion,
   #            extensibleMatch [9] MatchingRuleAssertion }
   #
   filtertype = getTag();
   filterlength = getL();
   filterend = hexptr+filterlength*2;
   if (filtertype == "a0") {  # and
       printf "(&";
       while(hexptr < filterend) {
           printFilter();
       }
       printf ")";
   } else {
     if (filtertype == "a1") {  # or
        printf "(|";
        while(hexptr < filterend) {
           printFilter();
        }
        printf ")";
     } else {
       if (filtertype == "a2") {  #  not
          printf "(!";
          printFilter();
          printf ")";
       } else {
         if (filtertype == "a3") {  #  equalityMatch
            printf "(" getString() "=" getString() ")" ;
         } else {
           if (filtertype == "87") {  #  presence
              hexptr -= 4;  # back to before tag
              printf "present: " getString();
           }
         }
       }
     }
   }
}

function printAttributes(attrlength, attrend) {
   #    AttributeDescriptionList ::= SEQUENCE OF
   #            AttributeDescription
   #
   #    AttributeDescription ::= LDAPString
   #
   hexptr += 2;  # skip tag
   attrlength = getL();
   attrend = hexptr+attrlength*2;
   printf " attributes: ";
   if (attrlength == 0) {   # all attributes
       printf "<ALL>";
   } else {
        while(hexptr < attrend) {
           printf getString() " ";
        }
   }   
}

function getTag(retVal) {
   #
   # get tag value from hex and hexptr
   #
   retVal = substr(hex,hexptr,2);
   hexptr += 2;    # skip tag
   return retVal;
}

function getL(retVal) {
   #
   # get length from hex and hexptr
   #
   if (substr(hex,hexptr,1) > 7) {
       n = substr(hex,hexptr+1,1);
       retVal = hex2dec(substr(hex,hexptr+2,2*n));
       hexptr += 2*n;
   } else {
       retVal = hex2dec(substr(hex,hexptr,2));
   }
   hexptr += 2;
   return retVal;
}

function getInt(size,retVal) {
   #
   # get integer from hex and hexptr
   #
   hexptr += 2;  # skip tag
   size = getL();  # size of integer
   retVal=hex2dec(substr(hex,hexptr,size*2));
   hexptr += size*2;  # skip integer
   return retVal;
}

function getBool(retVal) {
   #
   # get boolean from hex and hexptr
   #
   hexptr += 4;   # skip tag & length
   if (substr(hex,hexptr+1,1) == 0) {
        retVal = "false"
   } else {
        retVal = "true"
   }
   hexptr += 2;  # skip boolean
   return retVal;
}

function hex2dec(x,   retVal, L, i, j, h2d) {
   h2d = "0123456789ABCDEFabcdef";
   retVal = 0;
   L = length(x);
   for(i = 1; i <= L; i++) {
      j = index(h2d, substr(x, i, 1));
      if(j > 16) { j -= 6; }
      retVal = 16 * retVal + j - 1;
   }
   return retVal;
}
