#!/usr/bin/perl

# rebuildIPpackets.v3.pl
#
# Description:
#   Reconstructs packets of an IP trace for parsing
# Note:
#   This version uses RAM instead of temporary files. Though
#   unlikely, the worst-case RAM usage is the size of the
#   STDIN feed (ie: the first packet is continued at the
#   end of the feed).

$DEBUG=0;

undef $ipID;

while($line=<STDIN>) {
   $totalText.=$line if(!$ipID);

   if($line=~/ip_len=(\d+), ip_id=(\d+)/) {
      $ipID=$2;
      $ipLen=$1;
      print "DEBUG: found ipID: $ipID\n" if($DEBUG);

      # Mark if this packet is continued or not 
      $contHash{$ipID}=1 if($ipLen==1500);

      # See if this is a continuation packet
      # If it isn't set the reference to itself
      if(!$contHash{$ipID-1}) {
         $refHash{$ipID}=$ipID;
         push @ipidList, $ipID;

         $rebuildHash{$ipID}=$totalText;

      # If it is, reference back to the first packet
      } elsif($contHash{$ipID-1}) {
         # If so, mark the first packet
         $refHash{$ipID}=$refHash{$ipID-1};
      }

      # If this is the end, mark it as such
      $endHash{$refHash{$ipID}}=1 if(!$contHash{$ipID});

      print "DEBUG: $ipID is continued\n" if($DEBUG && $contHash{$ipID});
      print "DEBUG: $ipID is the end\n" if($DEBUG && $endHash{$ipID});

      next;
   }

   # We found an empty line - end of packet marker
   if($line=~/^$/) {
      undef $ipID;
      $totalText="";

      # See if we can't consolidate and free up RAM
      while($#ipidList>=0) {
         $testIpID=shift @ipidList;
         print "DEBUG: cleaning up: $testIpID\n" if($DEBUG);
         if($endHash{$testIpID}) {
            print $rebuildHash{$testIpID};
            print "\n";
            $rebuildHash{$testIpID}=undef;
         } else {
            unshift @ipidList, $testIpID;
            last;
         }
      }

      next;
   }

   # If we're a continuation, only print the data
   if($contHash{$ipID-1}) {
      if($line=~/^000/) {
         $line=~s/^00000/00001/;
         $rebuildHash{$refHash{$ipID}}.=$line;
      }
   } else {
      $rebuildHash{$refHash{$ipID}}.=$line;
   }

}

foreach $testIpID (@ipidList) {
   print $rebuildHash{$testIpID};
   print "\n";
   $rebuildHash{$testIpID}=undef;
}
