#!/usr/bin/perl

# perfmon_cpu.pl
#
# Last Updated: 2004/07/29 1036 CDT
# Desription:
#   Perl script to parse vmstat/iostat output into a graphical view
# Usage:
# AIX & Linux:
#    vmstat <INTERVAL> | ./perfmon_cpu.pl
# Solaris:
#    iostat <INTERVAL> | ./perfmon_cpu.pl
#
#   where <INTERVAL> is the sample size in seconds, such as 30 or 60

use POSIX;

# DEBUG
$DEBUG=0;

# unbuffer STDIN
select((select(STDIN), $|=1)[0]);
# unbuffer STDOUT
select((select(STDOUT), $|=1)[0]);

# sample vmstat - AIX
# r  b   avm   fre  re  pi  po  fr   sr  cy  in   sy  cs us sy id wa
# 0  0 1244420 5632449   0   0   0   0    0   0 2417 4777 3404  1  1 97  1

# sample iostat - Solaris
#   tty        fd0           sd6           ssd0          nfs1           cpu
# tin tout kps tps serv  kps tps serv  kps tps serv  kps tps serv   us sy wt id
#   0    6   0   0    0    0   0    0   42   3   47    0   0    0    2  1  1 96

# sample vmstat - Linux
#   procs                      memory    swap          io     system         cpu
# r  b  w   swpd   free   buff  cache  si  so    bi    bo   in    cs  us  sy  id
# 0  0  0  16320   3824  39664  29200   0   0     1     3    0     0   0   0   3


while($line=<STDIN>) {
   if(!$ostype) {
      $ostype="solaris" if($line=~/^\s*tty/);
      $ostype="linux" if($line=~/^\s*procs/);
      $ostype="aix" if($line=~/^\s*r\s+b\s+avm/);
   }
   next if($line!~/^\s*\d+/);

   print $line if($DEBUG);

   $line=~s/^\s*//;
   @logparts=split(/\s+/,$line);
   if($ostype eq "aix") {
      $user=$logparts[13];
      $system=$logparts[14];
      $idle=$logparts[15];
      $wait=$logparts[16];
   } elsif($ostype eq "solaris") {
      $user=$logparts[14];
      $system=$logparts[15];
      $idle=$logparts[16];
      $wait=$logparts[17];
   } elsif($ostype eq "linux") {
      $user=$logparts[13];
      $system=$logparts[14];
      $idle=$logparts[15];
      $wait=$logparts[16];
   }

   $remainder=100-($system + $user + $wait);
   $idle=$remainder if($idle>$remainder);

   print strftime("%Y%m%d %H:%M:%S",localtime);
   print ' |';
   print 'S' x $system;
   print 'U' x $user;
   print 'W' x $wait;
   print ' ' x $idle;
   print 'x' x (100-($system + $user + $wait + $idle));
   print "|";
   print " $user $system $idle $wait" if($DEBUG);
   print "\n";

}
