#!/bin/sh

# perftune_itim45.sh
#
# Last UpdateD: 2004/01/23 0934 CST
# Desription:
#   Shell script to tune the ITIM instance.
#
#   NOTE: This _will_ need to be customized for your environment.
#   The values shown below are to be used as guidelines only. Please
#   consult the ITIM Performance and Tuning Guide for more accurate
#   information.
# Usage:
#   This script should be run as the admin for the database (ie: one
#   that has connect and admin privileges).

# After customizing, remove the following two lines
echo "This script should be customized before running!"
exit;

DATABASE=itim
SCHEMA=ENROLE

db2 connect to $DATABASE

echo Updating db configuration parameters
db2 update database configuration for $DATABASE using dbheap 1200
db2 update database configuration for $DATABASE using applheapsz 2048
db2 update database configuration for $DATABASE using maxappls 125
db2 update database configuration for $DATABASE using app_ctl_heap_sz 1024
db2 update database configuration for $DATABASE using logsecond 12
db2 update database configuration for $DATABASE using logfilsiz 10000

echo Setting bufferpools
# Allocate 75MB to ibmdefaultbp; 225MB to enrolebp; and the rest to the system
db2 alter bufferpool ibmdefaultbp size 18311
db2 alter bufferpool enrolebp size 6867

echo Display bufferpool settings
db2 "select bpname,npages,pagesize from syscat.bufferpools"

#echo Adding more tablespaces
#[[ ! -f /home/db2inst1/db2inst1/NODE0000/SQL00001/enrole_data4 ]] && db2 "ALTER TABLESPACE ENROLE_DATA ADD (FILE '/home/db2inst1/db2inst1/NODE0000/SQL00001/enrole_data2' 16384, FILE '/home/db2inst1/db2inst1/NODE0000/SQL00001/enrole_data3' 16384, FILE '/home/db2inst1/db2inst1/NODE0000/SQL00001/enrole_data4' 16384) PREFETCHSIZE 16 OVERHEAD 24.1 TRANSFERRATE 0.9 BUFFERPOOL ENROLEBP"

echo Creating Indexes
db2 "CREATE UNIQUE INDEX ACTIVITY_ID_X ON $SCHEMA.ACTIVITY (\"ID\" ASC) INCLUDE (\"SUBTYPE\")"
db2 "CREATE INDEX ACTIVITY_PID_X ON $SCHEMA.ACTIVITY (\"PROCESS_ID\" DESC)" # from Jeff
db2 "CREATE INDEX ACTIVITY_STATE_X ON $SCHEMA.ACTIVITY (\"STATE\" DESC)" # from Jeff
db2 "CREATE UNIQUE INDEX PROCESS_ID_X ON $SCHEMA.PROCESS (\"ID\" ASC)"
db2 "CREATE INDEX PROCESSDATA_PID_X ON $SCHEMA.PROCESSDATA (\"PROCESS_ID\" DESC, \"DEF_ID\" DESC)"
db2 "CREATE INDEX WORKITEM_PID_X ON $SCHEMA.WORKITEM (\"PROCESS_ID\" DESC)"
db2 "CREATE INDEX PROCESS_SUB_X ON $SCHEMA.PROCESS (\"SUBMITTED\" DESC, \"PARENT_ID\" ASC)" # for filtered Pending and Completed Transactions

echo Setting DB2_RR_TO_RS=YES
db2set DB2_RR_TO_RS=YES

echo Restarting DB2
db2 terminate
db2 force applications all
sleep 3
db2stop
db2start

echo Performing runstats on tables to update index statistics
./perftune_runstats.sh itim ENROLE

