#!/bin/sh

# perftune_runstats.sh
#
#
#
# Last Updated: 2004/04/01 1416 CST
# Desription:
#   Shell script to run the db2 runstats command on one or more tables
#   for a given database and schema. It autodetects the db2 version using
#   db2level and passes the appropriate string to runstats to allow writes
#   to occur during the runstats.
# Usage:
#   This script should be run as the user for the database (ie: one that
#   has connect and runstats abilities and permissions).
#
# ./perftune_runstats.sh [DATABASE] [SCHEMA] [TABLES]
#    DATABASE = database to run against (optional - default ldapdb2)
#      SCHEMA = schema to run against (optional - default LDAPDB2)
#      TABLES = list of tables to run against, if none are given, all
#               tables are processed
#
# Note: If SCHEMA is given, DATABASE must be given. Likewise if TABLES
#   are specified, SCHEMA and DATABASE must be given.


# Detect arguments DATABASE and SCHEMA
if [ $# -gt 1 ]; then
  DATABASE=$1
  SCHEMA=`echo $2 | tr 'a-z' 'A-Z'`
  shift; shift
elif [ $# -gt 0 ]; then
  DATABASE=$1
  SCHEMA=LDAPDB2
  shift
else
  DATABASE=ldapdb2
  SCHEMA=LDAPDB2
fi

# Get any tables specified on the command line
if [ $# -gt 0 ]; then
  TABLES=$*
fi

# Options for runstats command
OPTIONS="with distribution and detailed indexes all"

# Find out DB2 version for runstats syntax
if db2level | grep "DB2 v7" > /dev/null; then
  ACCESS="shrlevel change"
  echo Detected DB2 major version 7
elif db2level | grep "DB2 v8" > /dev/null; then
  ACCESS="allow write access"
  echo Detected DB2 major version 8
fi

# Connect to the database
echo Connecting to $DATABASE
db2 connect to $DATABASE

# Execute Runstats either on all ables or
# just the ones specified on the command line
if [ "x$TABLES" = "x" ]; then
   # Execute runstats on all tables
   echo Performing runstats on all tables for schema $SCHEMA
   echo "  with options: $OPTIONS $ACCESS"
   for i in `db2 connect to $DATABASE > /dev/null; db2 list tables for all | \
      grep $SCHEMA | cut -d' ' -f1`
   do
      echo Table: $SCHEMA.$i
      db2 runstats on table $SCHEMA.$i $OPTIONS $ACCESS
   done
else
   # Execute runstats on all tables
   echo Performing runstats on tables: $TABLES for schema $SCHEMA
   echo "  with options: $OPTIONS $ACCESS"
   for i in $TABLES
   do
      echo Table: $SCHEMA.$i
      db2 runstats on table $SCHEMA.$i $OPTIONS $ACCESS
   done
fi

# If the database we're tuning is an LDAP database,
# update LDAP_DESC and LDAP_ENTRY stats in the statistics table
#if [ "$SCHEMA" = "LDAPDB2" ]; then
#  echo Updating LDAP_DESC and LDAP_ENTRY stats in the statistics table
#  db2 "update sysstat.tables set card = 9E18 where tabname = 'LDAP_DESC'"
#  db2 "update sysstat.tables set card = 9E18 where tabname = 'LDAP_ENTRY'"
#fi

